package vsp.gaussian;

import java.util.Random;

/**
 * Random numbers generator
 * 
 * @author Jan Tichava <jtichava@students.zcu.cz>
 *
 */
public class RandomGenerator {
	private static final double ROUND = 1000.0;
	private static final int STEPS = 20;
	private Params params;
	private Random random;
	private Gaussian gaussian;
	private Box box;
	private Histogram histogram;
	private double[] nums;
	private double sum;
	
	/**
	 * Creates generator object
	 * @param params parsed command line arguments
	 */
	public RandomGenerator(Params params) {
		this.params = params;
		this.nums = null;
		this.random = new Random();
		this.gaussian = new Gaussian(params);
		this.sum = -1.0;
		this.histogram = new Histogram(params, STEPS);
		this.box = new Box(
							params.getMedian() - params.getDeviation(),
							params.getMedian() + params.getDeviation(),
							gaussian.functionValue(params.getMedian())
						);
	}
	
	/**
	 * Generate random numbers
	 * @return array with random numbers
	 * @throws Exception
	 */
	public double[] generate() throws Exception {
		return generate(false);
	}
	
	/**
	 * Generate random numbers
	 * 
	 * @param silent
	 * @return array with random numbers
	 * @throws Exception
	 */
	public double[] generate(boolean silent) throws Exception {
		nums = new double[params.getCount()];
		sum = 0;
		
		for(int i = 0; i < params.getCount(); i++) {
			nums[i] = generateNumber();
			sum += nums[i];
			histogram.addValue(nums[i]);
		}
		
		if (!silent) {
			printStatistics(params);
		}
		
		return nums;
	}
	
	/**
	 * Get real median from generated numbers
	 * @return
	 * @throws Exception
	 */
	public double getRealMedian() throws Exception {
		if (sum < 0) {
			throw new Exception("Generate numbers first!");
		}
		
		return sum / params.getCount();
	}
	
	/**
	 * Get real deviation from generated numbers
	 * @return
	 * @throws Exception
	 */
	public double getRealDeviation() throws Exception {
		if (nums == null) {
			throw new Exception("Generate numbers first!");
		}
		
		double squareSum = 0;
		double realMedian = this.getRealMedian();
		
		for (double i : nums) {
			squareSum += Math.pow(realMedian - i, 2);
		}

		return (( 1.0 / ( params.getCount() - 1) ) * squareSum);
	}
	
	/**
	 * Generate next random number
	 * @return random number
	 */
	private double generateNumber() {
		while (true) {
			double xk = box.getA() + (box.getB() - box.getA()) * random.nextDouble();
			double zk = box.getC() * random.nextDouble();
			
			if (zk < gaussian.functionValue(xk)) {
				return xk;
			}
		}
	}
	
	/**
	 * Print statistics of generation
	 * @throws Exception
	 */
	private void printStatistics(Params params) throws Exception {
		System.out.println("E_teorie=" + params.getMedian());
		System.out.println("D_teorie=" + params.getDeviation());
		System.out.println("E_vypocet=" + round(this.getRealMedian()));
		System.out.println("D_vypocet=" + round(this.getRealDeviation()));
		System.out.println(histogram);
	}
	
	
	/**
	 * Round number to three digits 
	 * @param num common double number
	 * @return double with max three digits
	 */
	private static double round(double num) {
		return Math.round(num * ROUND) / ROUND;
	}
}
