package vsp;

import vsp.gaussian.Params;
import cz.zcu.fav.kiv.jsim.JSimInvalidParametersException;
import cz.zcu.fav.kiv.jsim.JSimLink;
import cz.zcu.fav.kiv.jsim.JSimProcess;
import cz.zcu.fav.kiv.jsim.JSimSecurityException;
import cz.zcu.fav.kiv.jsim.JSimSimulation;
import cz.zcu.fav.kiv.jsim.JSimSimulationAlreadyTerminatedException;
import cz.zcu.fav.kiv.jsim.JSimSystem;
import cz.zcu.fav.kiv.jsim.JSimTooManyProcessesException;

/**
 * 
 * Server class
 * 
 * @author Jan Tichava <jtichava@students.zcu.cz>
 *
 */
public class Server extends JSimProcess {

	/**
	 * Mean value
	 */
	private double ts;
	/**
	 * Assigned queue
	 */
	private QueueWithServer queue;
	
	/**
	 * Transactions counter
	 */
	private int counter;
	/**
	 * Real Tq
	 */
	private double transTq;

	/**
	 * Output queue 1
	 */
	private QueueWithServer output1;
	/**
	 * Output queue 2
	 */
	private QueueWithServer output2;
	/**
	 * Probability for queue 1
	 */
	private double probability1;
	/**
	 * Effective working time
	 */
	private double effectiveTime;

	/**
	 * Median for Gaussian 
	 */
	private double median;
	/**
	 * Deviation for Gaussian
	 */
	private double deviation;
	
	/**
	 * Silent mode for generating Gaussian
	 */
	private static final boolean SILENT = true;
	/**
	 * Is exp or Gauss?
	 */
	private boolean gauss;
	
	/**
	 * Creates new server
	 * 
	 * @param name server name
	 * @param simulation simulation object
	 * @param ts mean service time
	 * @param queue assigned queue
	 * @param output1 output queue 1
	 * @param output2 output queue 2
	 * @param probability1 probability for queue 1
	 * @throws JSimSimulationAlreadyTerminatedException
	 * @throws JSimInvalidParametersException
	 * @throws JSimTooManyProcessesException
	 */
	public Server(String name, JSimSimulation simulation, double ts, QueueWithServer queue, QueueWithServer output1, QueueWithServer output2, double probability1) throws JSimSimulationAlreadyTerminatedException, JSimInvalidParametersException, JSimTooManyProcessesException {
		super(name, simulation);
		
		this.ts = ts;
		this.queue = queue;
		this.output1 = output1;
		this.output2 = output2;
		this.probability1 = probability1;
		
		counter = 0;
		transTq = 0.0;
		effectiveTime = 0;
		
		gauss = false;
	}

	/**
	 * Creates new server
	 * 
	 * @param name server name
	 * @param simulation simulation object
	 * @param median median
	 * @param deviation deviation
	 * @param queue assigned queue
	 * @param output1 output queue 1
	 * @param output2 output queue 2
	 * @param probability1 probability for queue 1
	 * @throws JSimSimulationAlreadyTerminatedException
	 * @throws JSimInvalidParametersException
	 * @throws JSimTooManyProcessesException
	 */
	public Server(String name, JSimSimulation simulation, double median, double deviation, QueueWithServer queue, QueueWithServer output1, QueueWithServer output2, double probability1) throws JSimSimulationAlreadyTerminatedException, JSimInvalidParametersException, JSimTooManyProcessesException {
		super(name, simulation);
		
		this.median = median;
		this.deviation = deviation;
		this.queue = queue;
		this.output1 = output1;
		this.output2 = output2;
		this.probability1 = probability1;
		
		counter = 0;
		transTq = 0.0;
		effectiveTime = 0;
		
		gauss = true;
	}

	
	/**
	 * In infinite loop:
	 * - checks if queue is not empty
	 * - "work"
	 * - removes transaction from queue
	 * - decides to which output queue put transaction
	 * 
	 */
	protected void life() {
		
		try {
			while(true) {
				if (queue.empty()) {
					passivate();
				}
				else {
					double mu = 1 / ts;
					
					double workTime = 0;
					if (gauss) {
						Params params = new Params(1, median, deviation);
						vsp.gaussian.RandomGenerator generator = new vsp.gaussian.RandomGenerator(params);
						workTime = Math.abs(generator.generate(SILENT)[0]);
					}
					else {
						workTime = JSimSystem.negExp(mu);
					}
					
					hold(workTime);
					effectiveTime += workTime;
					
					JSimLink link = queue.first();

					Transaction t = (Transaction) link.getData();
					
					link.out();
					
					if (JSimSystem.uniform(0.0, 1.0) < probability1)
					{
						if (output1 != null) {
							link.into(output1);
							if (output1.getServer().isIdle()) {
								output1.getServer().activate(myParent.getCurrentTime());
							}
						}

						counter++;
						transTq += myParent.getCurrentTime() - t.getCreationTime();
					}
					else
					{
						if (output2 != null) {
							link.into(output2);
							if (output2.getServer().isIdle()) {
								output2.getServer().activate(myParent.getCurrentTime());
							}
						}
					}
					
					link = null;
					
				}
			}
		} catch (JSimSecurityException e) {
			e.printStackTrace();
		} catch (JSimInvalidParametersException e) {
			e.printStackTrace();
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	/**
	 * Get count of managed transactions
	 * 
	 * @return the counter
	 */
	public int getCounter() {
		return counter;
	}

	/**
	 * Get global Tq
	 * 
	 * @return the transTq
	 */
	public double getTransTq() {
		return transTq;
	}

	/**
	 * Get average Tq for one request
	 * 
	 * @return Tq
	 */
	public double getTq() {
		return transTq/counter;
	}
	
	/**
	 * Server load
	 * @return load
	 */
	public double getLoad() {
		return effectiveTime / myParent.getCurrentTime();
	}
}
