package vsp;

import java.text.DecimalFormat;
import java.text.NumberFormat;
import java.util.ArrayList;

/**
 * Collects statistics of stamp queue
 * 
 * @author Jan Tichava
 *
 */
public class Statistics extends Thread {

	/**
	 * Max stars in histogram
	 */
	private static final int MAX_STARS = 60;
	
	/**
	 * Watching queue
	 */
	private QueueWithServer queue;
	/**
	 * Scanning time
	 */
	private long scanTime;
	/**
	 * Should scan?
	 */
	private boolean run;
	/**
	 * Collected data
	 */
	private ArrayList<Long> lengths;
	
	/**
	 * Creates new object 
	 * 
	 * @param queue
	 * @param scanTime
	 */
	public Statistics(QueueWithServer queue, long scanTime) {
		this.queue = queue;
		this.scanTime = scanTime;
		this.run = true;
		this.lengths = new ArrayList<Long>();
	}
	
	/**
	 * Run collecting
	 */
	public void run() {
		System.out.println("Collecting statistics started...");
		while(run) {
			lengths.add(queue.cardinal());
			
			try {
				Thread.sleep(scanTime);
			} catch (InterruptedException e) {
				e.printStackTrace();
			}
		}
	}
	
	/**
	 * Stop collecting data
	 */
	public void stopCollecting() {
		run = false;
	}
	
	/**
	 * Get minimum queue length
	 * @return
	 */
	public long getMinLength() {
		long min = 0;

		for (Long l : lengths) {
			if (l < min) {
				min = l;
			}
		}
		
		return min;
	}
	
	/**
	 * Get maximum queue length
	 * @return
	 */
	public long getMaxLength() {
		long max = 0;

		for (Long l : lengths) {
			if (l > max) {
				max = l;
			}
		}
		
		return max;
	}
	
	/**
	 * Get queue's median
	 * @return
	 */
	public double getMedian() {
		double sum = 0.0;
	
		for (Long l : lengths) {
			sum += l;
		}
		
		return sum / lengths.size();
	}
	
	/**
	 * Get variance
	 * @return
	 */
	public double getVariance() {
		double median = getMedian();
		double sum = 0.0;
		
		for (Long l : lengths) {
			sum += Math.pow(l - median, 2);
		}
		
		return sum / lengths.size();
	}
	
	public double getSigma() {
		return Math.sqrt(getVariance());
	}


	/**
	 * Get queue name
	 * 
	 * @return the queue name
	 */
	public String getQueueName() {
		return queue.getHeadName();
	}
	
	/**
	 * Generates histogram data
	 * @return
	 */
	private long[] generateHistogram() {
		long min = getMinLength();
		long max = getMaxLength();
		int count = (int)(max - min) + 1;
		
		long[] histogram = new long[count];
		
		for (Long l : lengths) {
			histogram[(int)(l - min)]++;
		}
		
		return histogram;
	}
	
	/**
	 * Get histogram data
	 * @return
	 */
	public long[] getHistogramData() {
		return generateHistogram();
	}
	
	/**
	 * Get histogram in ASCII art
	 * @return
	 */
	public String getHistogramGraphics() {
		long[] histogram = generateHistogram();
		
		long max = 0;
		for (long l : histogram) {
			if (l > max) {
				max = l;
			}
		}
		NumberFormat twoDecimals = new DecimalFormat("00");
		NumberFormat threeDecimals = new DecimalFormat("000");
		
		
		String hist = "count in queue - real count - histogram\n";
		for (int i = 0; i < histogram.length; i++) {
			hist += twoDecimals.format(i) + " - " + threeDecimals.format(histogram[i]) + " - " + generateStars(histogram[i], max) + "\n";
		}
		
		return hist;
	}
	
	/**
	 * Generate set of stars for histogram
	 * @param value
	 * @param maxValue
	 * @return
	 */
	private String generateStars(long value, long maxValue) {
		
		int count = (int) (((value + 0.0) / maxValue) * MAX_STARS);
		
		String s = "";
		for (int i = 0; i < count; i++) {
			s += "*";
		}
		
		return s;
	}
}
